// this module attempts to make gulp able to provide help
// -> in 3.x, we can use the gulp-help package to do so
// -> in 4.x, this is shimmed in. In addition, we need to
//    - facilitate forward references, as per original gulp
//    -
const setTaskName = requireModule("set-task-name"),
  gulpVersion = requireModule("gulp-version");

if (gulpVersion.major === 3) {
  module.exports = require("gulp-help")(require("gulp"));
} else {
  const
    quieter = requireModule("reduce-gulp-noise");
  gulp = require("gulp"),
    help = {},
    FwdRef = require("undertaker-forward-reference");

  gulp.registry(new FwdRef());

  const
    originalTask = gulp.task,
    newTask = function () {
      let
        args = Array.from(arguments),
        taskName = args[0],
        helpMessage = "";
      if (typeof args[1] === "string") {
        helpMessage = args[1];
        args.splice(1, 1);
      }
      if (Array.isArray(args[1])) {
        const parallel = gulp.parallel(
          args[1].map(name => {
            // this allows late-overriding of tasks, as per assistance
            // at: https://github.com/gulpjs/gulp/issues/2337
            return setTaskName((...args) => gulp.series(name)(...args), quieter.markForSuppression(name)); // `${quieter.marker}(${name})`);
          }));
        setTaskName(parallel, `pre-${taskName}`);
        args[1] = gulp.series(
          parallel,
          args[2] || (() => Promise.resolve())
        );
        setTaskName(args[1], `[${taskName}]`);
        args.splice(2, 1);
      }
      help[taskName] = helpMessage;
      originalTask.call(gulp, taskName, args[1]);
      const generatedTask = originalTask.call(gulp, taskName);
      generatedTask.description = helpMessage;
      return generatedTask;
    };
  gulp.task = newTask.bind(gulp);
  gulp.task("help", () => {
    try {
      const chalk = require("ansi-colors"),
        green = chalk.greenBright.bind(chalk),
        yellow = chalk.yellowBright.bind(chalk),
        cyan = chalk.cyanBright.bind(chalk);

      console.log(yellow("Task help"));
      console.log(yellow(" - Tasks marked with * are affected by environment variables"));
      console.log(yellow(" - run the help:environment task for more info"));
      const keys = Object.keys(help).sort();
      const longestKeyLength = keys.reduce(function (acc, cur) {
        return cur.length > acc ? cur.length : acc;
      }, 0);
      keys.forEach(function (key) {
        if (!help[key]) {
          return console.log(cyan(key));
        }
        const helpMessage = help[key];
        while (key.length < longestKeyLength) {
          key += " ";
        }
        console.log(cyan(key) + "  " + green(helpMessage));
      });
      return Promise.resolve();
    } catch (e) {
      return Promise.reject(e);
    }
  });
  module.exports = gulp;
}

(function () {
  const
    env = requireModule("env"),
    gulp = module.exports;

  gulp.task("help:environment", () => {
    env.printHelp();
    return Promise.resolve();
  });
})();
